"""
_____________________________________________________________________.

:PROJECT: LARAsuite

*lara_django_sequences models *

:details: lara_django_sequences database models.
         -
:authors: mark doerr <mark.doerr@uni-greifswald.de>

.. note:: -
.. todo:: - remove unwanted models/fields
________________________________________________________________________
"""

from __future__ import annotations

import hashlib
import json
import uuid
from logging import getLogger
from typing import Any

from django.conf import settings
from django.db import models
from django.utils import timezone
from lara_django_base.models import BlockchainHashesAbstr, ExtraDataAbstr, ItemStatus, MediaType, Namespace, Tag
from lara_django_people.models import ItemSubsetAbstr

settings.FIXTURES += ["1_sequence_class_fix"]

logger = getLogger(__name__)


class BlockchainHashes(BlockchainHashesAbstr):
    """
    Store hashes of data for blockchain verification.

    This class can be used to store hashes of data, which are stored in a blockchain.
    The hashes can be used to verify the integrity of the data.
    The hashes are stored in a separate table, to avoid circular dependencies.
    """

    model_curi = "lara:data/BlockchainHashes"


class ExtraData(ExtraDataAbstr):
    """
    Extend Sequence data with extra information.

    This class can be used to extend the Sequence data, by extra information,
    e.g., properties relevant only for certain sequences.
    """

    model_curi = "lara:sequences/ExtraData"
    image = models.ImageField(
        upload_to="sequences/images",
        blank=True,
        null=True,
        help_text="extra sequence images",
    )
    file = models.FileField(upload_to="sequences/", blank=True, null=True, help_text="rel. path/filename")

    def save(self, *args: Any, **kwargs: Any) -> None:
        """Generate default values for name_full."""
        if self.version is None or self.version == "":
            self.version = "0.0.1"

        if self.hash_sha256 is None:
            if self.data_json is not None:
                to_hash = json.dumps(self.data_json)  # +  str(self.UUID) +
                self.hash_sha256 = hashlib.sha256(to_hash.encode("utf-8")).hexdigest()
            else:
                self.hash_sha256 = hashlib.sha256(str(self.extradata_id).encode("utf-8")).hexdigest()

        if self.name is None or self.name == "":
            if self.name_display is None or self.name_display == "":
                self.name = "_".join(
                    (
                        timezone.now().strftime("%Y%m%d_%H%M%S"),
                        "data",
                        self.hash_sha256[:8],
                    )
                )
            else:
                self.name = self.name_display.replace(" ", "_").lower().strip()

        else:
            self.name = self.name.replace(" ", "_").lower().strip()
        if self.name_full is None or self.name_full == "":
            if self.namespace is not None and self.version is not None:
                self.name_full = f"{self.namespace.uri}/" + "_".join((self.name.replace(" ", "_"), self.version))
            else:
                self.name_full = "_".join((self.name.replace(" ", "_"), self.version))

        if not self.iri:
            netloc = self.namespace.parse_uri().netloc
            path = self.namespace.parse_uri().path

            formatted_name = self.name.replace(" ", "_").lower().strip()
            self.iri = f"{settings.LARA_PREFIXES['lara']}sequences/ExtraData/{netloc}{path}/{formatted_name}"

        if self.datetime_created is None:
            self.datetime_created = timezone.now()

        if self.datetime_last_modified is None:
            self.datetime_last_modified = timezone.now()

        super().save(*args, **kwargs)


class SequenceClass(models.Model):
    """classes  of sequences, like  DNA, RNA, PNA, XNA, peptide/protein ..."""

    model_curi = "lara:sequences/SequenceClass"
    sequenceclass_id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    name = models.TextField(
        unique=True,
        help_text="classes  of sequences, like  DNA, RNA, PNA, XNA, peptide/protein ..",
    )
    # encoding should contain the number of bits and the mapping of the bits to the sequence
    # like: num_bits: 2 ; mapping : ["A", "C", "G", "T"] or ["A", "C", "G", "T", "U"]
    # for proteins it could be:
    # ["A", "C", "D", "E", "F", "G", "H", "I", "K", "L", "M", "N", "P", "Q", "R", "S", "T", "V", "W", "Y"]
    # the position in the list is the binary representation of the sequence
    encoding = models.JSONField(
        blank=True,
        null=True,
        help_text="encoding of the binary sequence, e.g. 4 bit, bit1: A, bit2: C, bit3: G, bit4: T",
    )
    iri = models.URLField(
        blank=True,
        null=True,
        unique=True,
        max_length=512,
        help_text="International Resource Identifier - IRI: is used for semantic representation ",
    )
    description = models.TextField(blank=True, null=True, help_text="description of the sequence class")

    def save(self, *args: Any, **kwargs: Any) -> None:
        """Generate default values for name."""
        if self.iri is None or self.iri == "":
            self.iri = (
                f"{settings.LARA_PREFIXES['lara']}sequences/SequenceClass/{self.name.replace(' ', '_').lower().strip()}"
            )

        super().save(*args, **kwargs)

    def __str__(self) -> str:
        """Return string representation of the sequence class."""
        return self.name or ""

    class Meta:
        """Meta configuration for SequenceClass model."""

        verbose_name_plural = "SequenceClasses"


class Sequence(models.Model):
    """Sequence class describing, sequences of DNA, RNA, PNA, XNA, peptide/protein ..."""

    model_curi = "lara:sequences/Sequence"
    sequence_id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    namespace = models.ForeignKey(
        Namespace,
        related_name="%(app_label)s_%(class)s_namespaces_related",
        related_query_name="%(app_label)s_%(class)s_namespaces_related_query",
        on_delete=models.CASCADE,
        null=True,
        blank=True,
        help_text="namespace of sequence",
    )
    name_display = models.TextField(
        blank=True,
        null=True,
        help_text="human readable title of sequence, can be different from name",
    )
    name = models.TextField(help_text="short sequence name for reference")
    hash_sha256 = models.CharField(
        max_length=256,
        blank=True,
        null=True,
        help_text="SHA256 hash of item for identity/duplicate checking",
    )
    hash_blockchain = models.CharField(
        max_length=512,
        blank=True,
        null=True,
        help_text="hash of sequence item for blockchain",
    )
    sequence_class = models.ForeignKey(
        SequenceClass,
        related_name="%(app_label)s_%(class)s_sequence_class_related",
        related_query_name="%(app_label)s_%(class)s_sequence_class",
        on_delete=models.CASCADE,
        blank=True,
        help_text="sequence class, like  DNA, RNA, PNA, XNA, peptide/protein ... ",
    )
    sequence = models.TextField(blank=True, null=True, help_text="sequence data as text")
    # sequence = models.BinaryField(
    #     blank=True,
    #     help_text="sequence can be any supported format, defined by substance "
    #               "(polymer) file type, binary, to save space",
    # )
    properties = models.JSONField(
        blank=True,
        null=True,
        help_text="properties of the sequence, e.g., length, GC content, UniProt Accession, NCBI Accession, PDB ID ...",
    )
    uniprot_accession = models.TextField(blank=True, null=True, help_text="UniProt Accession of the sequence")
    ncbi_gi = models.TextField(blank=True, null=True, help_text="NCBI GenInfo Identifier (GI)")
    ncbi_accession_version = models.TextField(
        blank=True, null=True, help_text="NCBI Accession Number.version identifier."
    )
    source_organism = models.JSONField(
        blank=True, null=True, help_text="organism from which the sequence originated - important for codon usage"
    )
    media_type = models.ForeignKey(
        MediaType,
        related_name="%(app_label)s_%(class)s_media_type_related",
        on_delete=models.CASCADE,
        null=True,
        blank=True,
        help_text="file type of sequence data (file)",
    )
    sequence_file = models.FileField(upload_to="sequences/", blank=True, null=True, help_text="rel. path/filename")

    status = models.ForeignKey(
        ItemStatus,
        related_name="%(app_label)s_%(class)s_status_related",
        on_delete=models.CASCADE,
        null=True,
        blank=True,
        help_text="status of sequence, e.g., selected, processed, synchronised ... ",
    )
    version = models.TextField(blank=True, null=True, help_text="version of the sequence")
    tags = models.ManyToManyField(
        Tag,
        blank=True,
        related_name="%(app_label)s_%(class)s_tags_related",
        related_query_name="%(app_label)s_%(class)s_tags",
        help_text="tags for sequence",
    )

    description = models.TextField(blank=True, null=True, help_text="description of the sequence")
    iri = models.URLField(
        blank=True,
        null=True,
        unique=True,
        max_length=512,
        help_text="International Resource Identifier - IRI: is used for semantic representation ",
    )
    datetime_last_modified = models.DateTimeField(
        blank=True, null=True, help_text="date and time when data was last modified"
    )

    class Meta:
        """Meta configuration for Sequence model."""

        # ~ verbose_name = 'ForumContribution'
        # ~ verbose_name_plural = 'ForumContributions'
        # db_table = "lara_metainfo_item_class"
        constraints = (
            models.UniqueConstraint(
                fields=["namespace", "name", "version"],
                name="%(app_label)s_%(class)s_name_namespace_version_unique",
            ),
        )

    def __str__(self) -> str:
        """Return string representation of the sequence."""
        return self.name or ""

    def __repr__(self) -> str:
        """Return string representation of the sequence."""
        return self.name or ""

    @property
    def encoding(self) -> dict[str, Any] | None:
        """Get the encoding from the sequence class."""
        if not hasattr(self, "_cached_encoding"):
            if self.sequence_class is None:
                logger.error("Sequence class with encoding is not defined")
                return None
            self._cached_encoding = self.sequence_class.encoding
        return self._cached_encoding

    @property
    def sequence_text(self) -> str | None:
        """Get the sequence as decoded text."""
        return self._decode_sequence(self.sequence)

    @sequence_text.setter
    def sequence_text(self, sequence_text: str) -> None:
        self._encode_sequence(sequence_text)

    def _encode_sequence(self, sequence_text: str) -> None:
        encoding = self.encoding
        if encoding is None:
            return

        num_bits = encoding["num_bits"]
        mapping_list = encoding["mapping"]
        mapping_dict = {mapping_list[i]: i for i in range(len(mapping_list))}
        sequence_binary = b""

        for char in sequence_text:
            sequence_binary += mapping_dict[char].to_bytes(num_bits, byteorder="big")

        self.sequence = sequence_binary

    def _encode_sequence_old(self, sequence_text: str) -> None:
        # used sequence_class.encoding to encode sequence

        if self.sequence_class is None:
            logger.error("Sequence class with encoding is not defined")
            return
        encoding = self.sequence_class.encoding
        # number of bits
        num_bits = encoding["num_bits"]
        # mapping of bits to sequence
        mapping_list = encoding["mapping"]

        # create a dictionary with the mapping
        mapping_dict = {mapping_list[i]: i for i in range(len(mapping_list))}

        # use the mapping to encode the sequence, each character in the sequence_text is mapped to num_bits bits
        sequence_binary = b""

        for char in sequence_text:
            sequence_binary += mapping_dict[char].to_bytes(num_bits, byteorder="big")

        self.sequence = sequence_binary

    def _decode_sequence(self, sequence_binary: bytes | None) -> str | None:
        encoding = self.encoding
        if encoding is None:
            return None

        num_bits = encoding["num_bits"]
        mapping_list = encoding["mapping"]
        mapping_dict = {i: mapping_list[i] for i in range(len(mapping_list))}
        sequence_text = ""

        for i in range(0, len(sequence_binary), num_bits):
            sequence_text += mapping_dict[int.from_bytes(sequence_binary[i : i + num_bits], byteorder="big")]

        return sequence_text

    def _decode_sequence_old(self, sequence_binary: bytes | None) -> str | None:
        # used sequence_class.encoding to encode sequence
        if self.sequence_class is None:
            logger.error("Sequence class with encoding is not defined")
            return None
        encoding = self.sequence_class.encoding
        # number of bits
        num_bits = encoding["num_bits"]
        # mapping of bits to sequence
        mapping_list = encoding["mapping"]

        # create a dictionary with the mapping
        mapping_dict = {i: mapping_list[i] for i in range(len(mapping_list))}

        # use the mapping to decode the sequence, each num_bits bits are mapped to a character in the sequence_text

        sequence_text = ""

        for i in range(0, len(sequence_binary), num_bits):
            sequence_text += mapping_dict[int.from_bytes(sequence_binary[i : i + num_bits], byteorder="big")]

        return sequence_text

    def save(self, *args: Any, **kwargs: Any) -> None:
        """Generate default values for IRIs and names."""
        if self.version is None or self.version == "":
            self.version = "0.0.1"

        if self.hash_sha256 is None:
            if self.sequence is not None:
                to_hash = json.dumps(self.sequence) + str(self.sequence_id)
                self.hash_sha256 = hashlib.sha256(to_hash.encode("utf-8")).hexdigest()
            else:
                self.hash_sha256 = hashlib.sha256(str(self.sequence_id).encode("utf-8")).hexdigest()

        if self.name is None or self.name == "":
            if self.name_display is None or self.name_display == "":
                self.name = "_".join(
                    (
                        timezone.now().strftime("%Y%m%d_%H%M%S"),
                        "sequence",
                        self.hash_sha256[:8],
                    )
                )
            else:
                self.name = self.name_display.replace(" ", "_").lower().strip()

        else:
            self.name = self.name.replace(" ", "_").lower().strip()

        if not self.iri:
            netloc = self.namespace.parse_uri().netloc
            path = self.namespace.parse_uri().path

            formatted_name = self.name.replace(" ", "_").lower().strip()
            self.iri = f"{settings.LARA_PREFIXES['lara']}sequences/Sequence/{netloc}{path}/{formatted_name}"

        if self.datetime_last_modified is None:
            self.datetime_last_modified = timezone.now()

        super().save(*args, **kwargs)


class SequencesSubset(ItemSubsetAbstr):
    """
    Store a subset of sequences for analysis or projects.

    This class can be used to store a subset of sequences, e.g., a selection of sequences,
    which are relevant for a certain analysis, project, publication.
    """

    model_curi = "lara:sequences/SequencesSubset"

    sequencessubset_id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)

    sequences = models.ManyToManyField(
        Sequence,
        related_name="%(app_label)s_%(class)s_sequences_related",
        related_query_name="%(app_label)s_%(class)s_sequences",
        help_text="sequences in the subset",
        through="OrderedSequencesSubset",
    )


class OrderedSequencesSubset(models.Model):
    """
    Through model for the many-to-many relationship between SequencesSubset and Sequence.

    This class can be used to store the order of sequences in a subset.
    """

    model_curi = "lara:sequences/OrderedSequencesSubset"

    orderedsequencessubset_id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)

    sequence = models.ForeignKey(
        Sequence,
        related_name="%(app_label)s_%(class)s_sequence_related",
        related_query_name="%(app_label)s_%(class)s_sequence",
        on_delete=models.CASCADE,
        help_text="sequence in the subset",
    )

    sequences_subset = models.ForeignKey(
        SequencesSubset,
        related_name="%(app_label)s_%(class)s_sequencessubset_related",
        related_query_name="%(app_label)s_%(class)s_sequencessubset",
        on_delete=models.CASCADE,
        help_text="subset of sequences",
    )

    order = models.IntegerField(help_text="order of sequence in subset")

    class Meta:
        """Meta configuration for OrderedSequencesSubset model."""

        constraints = (
            models.UniqueConstraint(
                fields=["sequence", "sequences_subset"],
                name="%(app_label)s_%(class)s_sequence_sequencessubset_unique",
            ),
        )
        ordering = ("order",)

    def save(self, *args: Any, **kwargs: Any) -> None:
        """Generate default values for order."""
        if self.order is None:
            self.order = 0

        super().save(*args, **kwargs)



"""
_____________________________________________________________________.

:PROJECT: LARAsuite

*lara_django_sequences admin *

:details: lara_django_sequences admin module admin backend configuration.
         -
:authors: mark doerr <mark.doerr@uni-greifswald.de>

.. note:: -
.. todo:: - run "lara-django-dev admin_generator lara_django_sequences >> admin.py" to update this file
________________________________________________________________________
"""

# -*- coding: utf-8 -*-
from django.contrib import admin

from .models import ExtraData, Sequence, SequenceClass


@admin.register(ExtraData)
class ExtraDataAdmin(admin.ModelAdmin):
    """Admin interface for ExtraData model."""

    list_display = (
        "name",
        "namespace",
        "data_type",
        "description",
        "extradata_id",
    )
    list_filter = ("data_type", "namespace", "media_type")


@admin.register(SequenceClass)
class SequenceClassAdmin(admin.ModelAdmin):
    """Admin interface for SequenceClass model."""

    list_display = (
        "name",
        "description",
        "sequenceclass_id",
    )
    search_fields = ("name",)


@admin.register(Sequence)
class SequenceAdmin(admin.ModelAdmin):
    """Admin interface for Sequence model."""

    list_display = (
        "name",
        "namespace",
        "sequence_class",
        "status",
        "description",
        "sequence_id",
    )
    list_filter = ("namespace", "sequence_class", "media_type", "status")
    raw_id_fields = ("tags",)
    search_fields = ("name",)



"""
_____________________________________________________________________.

:PROJECT: LARAsuite

*lara_django_sequences gRPC services*

:details: lara_django_sequences gRPC services.
         -
:authors: mark doerr <mark.doerr@uni-greifswald.de>

.. note:: -
.. todo:: -
________________________________________________________________________
"""

## generated with django-socio-grpc generateprpcinterface lara_django_sequences  (LARA-version)

from django_socio_grpc import generics, mixins
from lara_django_base.grpc.mixins import AsyncDownloadModelMixin, AsyncRetrieveByNameModelMixin, AsyncUploadModelMixin
from lara_django_sequences_grpc.v1 import lara_django_sequences_pb2

from lara_django_sequences.models import ExtraData, Sequence, SequenceClass, SequencesSubset

from ..filters import ExtraDataFilterSet, SequenceClassFilterSet, SequenceFilterSet, SequencesSubsetFilterSet
from .serializers import (
    ExtraDataProtoSerializer,
    SequenceClassProtoSerializer,
    SequenceFullProtoSerializer,
    SequenceProtoSerializer,
    SequencesSubsetProtoSerializer,
)


class ExtraDataService(
    generics.AsyncModelService,
    mixins.AsyncStreamModelMixin,
    AsyncUploadModelMixin,
    AsyncRetrieveByNameModelMixin,
    AsyncDownloadModelMixin,
):
    """Service for handling ExtraData model operations."""

    pb2 = lara_django_sequences_pb2
    queryset = ExtraData.objects.all()
    serializer_class = ExtraDataProtoSerializer
    filterset_class = ExtraDataFilterSet


class SequenceClassService(generics.AsyncModelService, mixins.AsyncStreamModelMixin):
    """Service for handling SequenceClass model operations."""

    queryset = SequenceClass.objects.all()
    serializer_class = SequenceClassProtoSerializer
    filterset_class = SequenceClassFilterSet


class SequenceService(
    generics.AsyncModelService,
    mixins.AsyncStreamModelMixin,
    AsyncUploadModelMixin,
    AsyncRetrieveByNameModelMixin,
    AsyncDownloadModelMixin,
):
    """Service for handling Sequence model operations."""

    pb2 = lara_django_sequences_pb2
    queryset = Sequence.objects.all()
    serializer_class = SequenceProtoSerializer
    filterset_class = SequenceFilterSet
    lookup_field = "sequence_id"


# class SequenceByNameService(generics.GenericService, mixins.AsyncListModelMixin, mixins.AsyncRetrieveModelMixin):
#     queryset = Sequence.objects.all()
#     serializer_class = SequenceByNameProtoSerializer
#     filterset_class = SequenceFilterSet
#     lookup_field = "name"


class SequenceFullService(
    generics.AsyncModelService,
    mixins.AsyncStreamModelMixin,
    AsyncUploadModelMixin,
    AsyncRetrieveByNameModelMixin,
    AsyncDownloadModelMixin,
):
    """Service for handling full Sequence model operations."""

    pb2 = lara_django_sequences_pb2
    queryset = Sequence.objects.all()
    serializer_class = SequenceFullProtoSerializer
    filterset_class = SequenceFilterSet
    search_fields = (
        "name",
        "description",
    )


class SequencesSubsetService(generics.AsyncModelService, mixins.AsyncStreamModelMixin):
    """Service for handling SequencesSubset model operations."""

    queryset = SequencesSubset.objects.all()
    serializer_class = SequencesSubsetProtoSerializer
    filterset_class = SequencesSubsetFilterSet
    search_fields = (
        "name",
        "description",
    )



"""
LARAsuite lara_django_sequences Model Context Protocol (MCP) handler.

:PROJECT: LARAsuite

*lara_django_sequences Model Context Protocol (MCP) handler *

:details: lara_django_sequences Model Context Protocol (MCP) handler.

:authors: mark doerr <mark.doerr@uni-greifswald.de>

________________________________________________________________________
"""

from mcp_server import ModelQueryToolset

from .models import (
    BlockchainHashes,
    ExtraData,
    OrderedSequencesSubset,
    Sequence,
    SequenceClass,
    SequencesSubset,
)


class BlockchainHashesQueryTool(ModelQueryToolset):
    """MCP query tool for BlockchainHashes model."""

    model = BlockchainHashes


class ExtraDataQueryTool(ModelQueryToolset):
    """MCP query tool for ExtraData model."""

    model = ExtraData


class SequenceClassQueryTool(ModelQueryToolset):
    """MCP query tool for SequenceClass model."""

    model = SequenceClass


class SequenceQueryTool(ModelQueryToolset):
    """MCP query tool for Sequence model."""

    model = Sequence


class SequencesSubsetQueryTool(ModelQueryToolset):
    """MCP query tool for SequencesSubset model."""

    model = SequencesSubset


class OrderedSequencesSubsetQueryTool(ModelQueryToolset):
    """MCP query tool for OrderedSequencesSubset model."""

    model = OrderedSequencesSubset



"""
_____________________________________________________________________.

:PROJECT: LARAsuite

*lara_django_sequences admin *

:details: lara_django_sequences admin module admin backend configuration.
         -
:authors: mark doerr <mark.doerr@uni-greifswald.de>

.. note:: -
.. todo:: - run "lara-django-dev tables_generator lara_django_sequences >> tables.py" to update this file
________________________________________________________________________
"""

# django Tests s. https://docs.djangoproject.com/en/4.1/topics/testing/overview/ for lara_django_sequences
# generated with django-extensions tests_generator  lara_django_sequences > tests.py (LARA-version)

import django_tables2 as tables

from .models import ExtraData, Sequence, SequenceClass


class ExtraDataTable(tables.Table):
    """Table for displaying ExtraData instances."""

    # adding link to column <column-to-be-linked>
    # <column-to-be-linked> = tables.Column(linkify=('lara_django_sequences:extradata-detail', [tables.A('pk')]))

    class Meta:  # noqa: D106
        model = ExtraData

        fields = (
            "data_type",
            "namespace",
            "URI",
            "data_json",
            "media_type",
            "iri",
            "URL",
            "description",
            "image",
            "file",
        )


class SequenceClassTable(tables.Table):
    """Table for displaying SequenceClass instances."""

    # adding link to column <column-to-be-linked>
    # <column-to-be-linked> = tables.Column(linkify=('lara_django_sequences:sequenceclass-detail', [tables.A('pk')]))

    class Meta:  # noqa: D106
        model = SequenceClass

        fields = ("name", "description")


class SequenceTable(tables.Table):
    """Table for displaying Sequence instances."""

    # adding link to column <column-to-be-linked>
    name = tables.Column(linkify=("lara_django_sequences:sequence-detail", [tables.A("pk")]))

    class Meta:  # noqa: D106
        model = Sequence

        fields = (
            "namespace",
            "name",
            "sequence_class",
            "sequence",
            "media_type",
            "sequence_file",
            "status",
            "description",
        )



"""
_____________________________________________________________________.

:PROJECT: LARAsuite

*lara_django_sequence semantics metadata handler *

:details: lara_django_sequence semantics metadata handler.
:authors: mark doerr <mark.doerr@uni-greifswald.de>

________________________________________________________________________
"""

from __future__ import annotations

import logging
from typing import TYPE_CHECKING, Any

from lara_django_base.tasks import semantic_data_to_triplestore

if TYPE_CHECKING:
    from django.db import models

logger = logging.getLogger("lara_semantics")


data_model_map = {
    "Data": "Data",
}


def get_semantic_data_map(instance: models.Model) -> dict[str, Any]:
    """
    Return dictionary with semantic mapping of the data.

    TODO: add more semantic mapping
    """
    return {
        "data_id": str(instance.data_id),  # hasLaraID
        "name_full": instance.name_full,
        "hasNamespace": instance.namespace.uri,
        "datetime_created": instance.datetime_created,
        "datetime_last_modified": instance.datetime_last_modified,
        "version": instance.version,
        "description": instance.description,
    }


def insert_semantic_data(sender: type[models.Model], instance: models.Model, **kwargs: Any) -> None:
    """
    Insert semantics data handler function.

    This function is called when a new data instance is created.
    It extracts the semantic data from the instance and sends it to the
    semantic_data_to_triplestore celery task for processing.
    """
    graph_urn = "urn:sparql:lara:data"

    try:
        data_model = data_model_map[instance.name]
    except KeyError:
        logger.exception("Error retrieving semantic data model")
        data_model = instance.name
    else:
        semantic_data_to_triplestore.delay(
            instance_name=instance.name,
            data_model=data_model,
            attributes=get_semantic_data_map(instance),
            graph_urn=graph_urn,
        )



"""
_____________________________________________________________________.

:PROJECT: LARAsuite

*lara_django_sequences urls *

:details: lara_django_sequences urls module.
         - add app specific urls here
         -
:authors: mark doerr <mark.doerr@uni-greifswald.de>

.. note:: -
.. todo:: -
________________________________________________________________________
"""

from django.urls import path
from django_socio_grpc.settings import grpc_settings

from . import views

# Add your {cookiecutter.project_slug}} urls here.


# !! this sets the apps namespace to be used in the template
app_name = "lara_django_sequences"

# companies and institutions should also be added
# the 'name' attribute is used in templates to address the url independent of the view


urlpatterns = [
    path("sequence/list/", views.SequenceSingleTableView.as_view(), name="sequence-list"),
    path("sequence/create/", views.SequenceCreateView.as_view(), name="sequence-create"),
    path("sequence/update/<uuid:pk>", views.SequenceUpdateView.as_view(), name="sequence-update"),
    path("sequence/delete/<uuid:pk>", views.SequenceDeleteView.as_view(), name="sequence-delete"),
    path("sequence/<uuid:pk>/", views.SequenceDetailView.as_view(), name="sequence-detail"),
    path("", views.SequenceSingleTableView.as_view(), name="sequence-root"),
    #     path('', views.EntitySingleTableView.as_view(), name='entity-list-root'),
    #     path('entities/list/', views.EntitySingleTableView.as_view(), name='entity-list'),
    #     path('addresses/list/', views.AddressesListView.as_view(), name='address-list'),
    #     path('addresses/create/', views.AddressCreateView.as_view(),
    #          name='address-create'),
    #     path('addresses/update/<uuid:pk>', views.AddressUpdateView.as_view(),
    #          name='address-update'),
    #     path('addresses/delete/<uuid:pk>', views.AddressDeleteView.as_view(),
    #          name='address-delete'),
]

# register handlers in settings
grpc_settings.user_settings["GRPC_HANDLERS"] += ["lara_django_sequences.grpc.handlers.grpc_handlers"]



"""
LARAsuite lara_django_myproj gRPC handlers.

:PROJECT: LARAsuite

*lara_django_myproj gRPC handlers*

:details: lara_django_myproj gRPC handlers.
         -
:authors: mark doerr <mark.doerr@uni-greifswald.de>

.. note:: -
.. todo:: -
________________________________________________________________________
"""

# generated with django-socio-grpc generateprpcinterface lara_django_sequences  (LARA-version)

# import logging
from django_socio_grpc.services.app_handler_registry import AppHandlerRegistry

from lara_django_sequences.grpc.services import (
    ExtraDataService,
    SequenceClassService,
    SequenceFullService,  # SequenceByNameService,
    SequenceService,
    SequencesSubsetService,
)


def grpc_handlers(server):
    """Register gRPC handlers for lara_django_sequences."""
    app_registry = AppHandlerRegistry("lara_django_sequences", server)

    app_registry.get_grpc_module = lambda: "lara_django_sequences_grpc.v1"

    app_registry.register(ExtraDataService)

    app_registry.register(SequenceClassService)

    app_registry.register(SequenceService)

    # app_registry.register(SequenceByNameService)

    app_registry.register(SequenceFullService)

    app_registry.register(SequencesSubsetService)



from importlib.metadata import version as get_version

__version__ = get_version(__package__)

__version_info__ = tuple(int(num) if num.isdigit() else num for num in __version__.replace("-", ".", 1).split("."))



import logging

from django.contrib.postgres.search import SearchVector
from django.db.models.signals import post_save
from django.dispatch import receiver
from lara_django_base.decorators import postgres_only

from .models import ExtraData

logger = logging.getLogger(__name__)


@receiver(post_save, sender=ExtraData)
@postgres_only
def update_search_vector(sender, instance, **kwargs):
    """
    Update the search vector for ExtraData instances after saving.

    This function is triggered by the post_save signal and updates the
    search_vector field of the ExtraData instance with a combination of
    SearchVector fields, each assigned a weight.
    """
    sender.objects.filter(pk=instance.pk).update(
        search_vector=(
            SearchVector("name", weight="A")
            + SearchVector("name_display", weight="B")
            + SearchVector("description", weight="C")
        )
    )


# @receiver(post_save, sender=Sequence)
# @postgres_only
# def update_search_vector(sender, instance, **kwargs):
#     sender.objects.filter(pk=instance.pk).update(
#         search_vector=(
#             SearchVector('name', weight='A') +
#             SearchVector('name_full', weight='B')
#         )
#     )

# @receiver(post_save, sender=Sequence)
# def semantics_data_handler(sender, instance, created, **kwargs):
#     """Semantic data post_save signal handler.

#        This function is triggered after an instance of the Evaluation model is saved.
#        It calls the insert_semantics function to handle the semantic data insertion
#        into the triplestore, running asynchronously as celery task.
#     """
#     logger.debug("post_save_handler1: %s, %s ", sender, instance)

#     insert_semantic_data(sender, instance, **kwargs)



"""
LARAsuite lara_django_myproj gRPC serializer.

:PROJECT: LARAsuite

*lara_django_myproj gRPC serializer*

:details: lara_django_myproj gRPC serializer.
         -
:authors: mark doerr <mark.doerr@uni-greifswald.de>

.. note:: -
.. todo:: -
________________________________________________________________________
"""

## generated with django-socio-grpc generateprpcinterface lara_django_sequences  (LARA-version)

from django_socio_grpc import proto_serializers
from lara_django_base.grpc.serializers import (
    ItemStatusProtoSerializer,
    MediaTypeProtoSerializer,
    NamespaceProtoSerializer,
)
from lara_django_base.models import DataType, ItemStatus, MediaType, Namespace, Tag
from lara_django_people.models import Entity, Group
from lara_django_sequences_grpc.v1 import lara_django_sequences_pb2
from rest_framework.serializers import PrimaryKeyRelatedField, UUIDField

from lara_django_sequences.models import ExtraData, Sequence, SequenceClass, SequencesSubset


class ExtraDataProtoSerializer(proto_serializers.ModelProtoSerializer):
    """Proto serializer for ExtraData model."""

    data_type = PrimaryKeyRelatedField(
        queryset=DataType.objects.all(), pk_field=UUIDField(format="hex_verbose"), required=False, allow_null=True
    )
    namespace = PrimaryKeyRelatedField(queryset=Namespace.objects.all(), pk_field=UUIDField(format="hex_verbose"))
    media_type = PrimaryKeyRelatedField(
        queryset=MediaType.objects.all(), pk_field=UUIDField(format="hex_verbose"), required=False, allow_null=True
    )

    class Meta:
        """Meta configuration for ExtraDataProtoSerializer."""

        model = ExtraData
        proto_class = lara_django_sequences_pb2.ExtraDataResponse

        proto_class_list = lara_django_sequences_pb2.ExtraDataListResponse

        # [data_type', namespace', URI', text', XML', JSON', bin', media_type',
        # IRI', URL', description', image', file']
        fields = "__all__"


class SequenceClassProtoSerializer(proto_serializers.ModelProtoSerializer):
    """Proto serializer for SequenceClass model."""

    class Meta:
        """Meta configuration for SequenceClassProtoSerializer."""

        model = SequenceClass
        proto_class = lara_django_sequences_pb2.SequenceClassResponse

        proto_class_list = lara_django_sequences_pb2.SequenceClassListResponse

        fields = "__all__"  # [name', description']


class SequenceProtoSerializer(proto_serializers.ModelProtoSerializer):
    """Proto serializer for Sequence model."""

    namespace = PrimaryKeyRelatedField(queryset=Namespace.objects.all(), pk_field=UUIDField(format="hex_verbose"))
    sequence_class = PrimaryKeyRelatedField(
        queryset=SequenceClass.objects.all(), pk_field=UUIDField(format="hex_verbose")
    )
    media_type = PrimaryKeyRelatedField(
        queryset=MediaType.objects.all(), pk_field=UUIDField(format="hex_verbose"), required=False, allow_null=True
    )
    status = PrimaryKeyRelatedField(
        queryset=ItemStatus.objects.all(), pk_field=UUIDField(format="hex_verbose"), required=False, allow_null=True
    )
    tags = PrimaryKeyRelatedField(
        queryset=Tag.objects.all(), pk_field=UUIDField(format="hex_verbose"), many=True, required=False, allow_null=True
    )

    class Meta:
        """Meta configuration for SequenceProtoSerializer."""

        model = Sequence
        proto_class = lara_django_sequences_pb2.SequenceResponse

        proto_class_list = lara_django_sequences_pb2.SequenceListResponse

        # [namespace', name', name_full', UUID', hash_sha256', sequence_class',
        # sequence', media_type', sequence_file', status', description', URI', IRI']
        fields = "__all__"


class SequenceByNameProtoSerializer(proto_serializers.ModelProtoSerializer):
    """Proto serializer for Sequence model by name."""

    namespace = PrimaryKeyRelatedField(queryset=Namespace.objects.all(), pk_field=UUIDField(format="hex_verbose"))
    sequence_class = PrimaryKeyRelatedField(
        queryset=SequenceClass.objects.all(), pk_field=UUIDField(format="hex_verbose")
    )
    media_type = PrimaryKeyRelatedField(
        queryset=MediaType.objects.all(), pk_field=UUIDField(format="hex_verbose"), required=False, allow_null=True
    )
    status = PrimaryKeyRelatedField(
        queryset=ItemStatus.objects.all(), pk_field=UUIDField(format="hex_verbose"), required=False, allow_null=True
    )
    tags = PrimaryKeyRelatedField(
        queryset=Tag.objects.all(), pk_field=UUIDField(format="hex_verbose"), many=True, required=False, allow_null=True
    )

    class Meta:
        """Meta configuration for SequenceByNameProtoSerializer."""

        model = Sequence
        proto_class = lara_django_sequences_pb2.SequenceResponse

        proto_class_list = lara_django_sequences_pb2.SequenceListResponse

        # [namespace', name', name_full', UUID', hash_sha256', sequence_class',
        # sequence', media_type', sequence_file', status', description', URI', IRI']
        fields = "__all__"


class SequencesSubsetProtoSerializer(proto_serializers.ModelProtoSerializer):
    """Proto serializer for SequencesSubset model."""

    namespace = PrimaryKeyRelatedField(queryset=Namespace.objects.all(), pk_field=UUIDField(format="hex_verbose"))
    entity = PrimaryKeyRelatedField(queryset=Entity.objects.all(), pk_field=UUIDField(format="hex_verbose"))
    group = PrimaryKeyRelatedField(
        queryset=Group.objects.all(), pk_field=UUIDField(format="hex_verbose"), required=False, allow_null=True
    )
    sequences = PrimaryKeyRelatedField(
        queryset=Sequence.objects.all(), pk_field=UUIDField(format="hex_verbose"), many=True
    )
    tags = PrimaryKeyRelatedField(
        queryset=Tag.objects.all(), pk_field=UUIDField(format="hex_verbose"), many=True, required=False, allow_null=True
    )

    class Meta:
        """Meta configuration for SequencesSubsetProtoSerializer."""

        model = SequencesSubset
        proto_class = lara_django_sequences_pb2.SequencesSubsetResponse

        proto_class_list = lara_django_sequences_pb2.SequencesSubsetListResponse

        fields = "__all__"


class SequenceFullProtoSerializer(proto_serializers.ModelProtoSerializer):
    """Proto serializer for full Sequence model with nested serializers."""

    namespace = NamespaceProtoSerializer(read_only=True)
    sequence_class = SequenceClassProtoSerializer(read_only=True)
    media_type = MediaTypeProtoSerializer(read_only=True)
    status = ItemStatusProtoSerializer(read_only=True)

    class Meta:
        """Meta configuration for SequenceFullProtoSerializer."""

        model = Sequence
        proto_class = lara_django_sequences_pb2.SequenceFullResponse

        proto_class_list = lara_django_sequences_pb2.SequenceFullListResponse

        # [namespace', name', name_full', UUID', hash_sha256', sequence_class',
        # sequence', media_type', sequence_file', status', description', URI', IRI']
        fields = "__all__"



"""
LARAsuite lara_django_sequences admin forms.

:PROJECT: LARAsuite

*lara_django_sequences admin*

:details: lara_django_sequences admin module backend form configuration.
         -
:authors: mark doerr <mark.doerr@uni-greifswald.de>

.. note:: -
.. todo:: - run "lara-django-dev forms_generator -c lara_django_sequences > forms.py" to update this file
________________________________________________________________________
"""

# django crispy forms s. https://github.com/django-crispy-forms/django-crispy-forms for []
# generated with django-extensions forms_generator -c  [] > forms.py (LARA-version)

from typing import ClassVar

from django import forms
from django.forms.renderers import TemplatesSetting

from .models import ExtraData, Sequence, SequenceClass


class CustomFormRenderer(TemplatesSetting):
    """Custom renderer using app-specific field-group template."""

    field_template_name = "lara_django_sequences/forms/field_group.html"


class SearchableSelectMultiple(forms.SelectMultiple):
    """Custom widget for searchable multi-select."""

    template_name = "lara_django_sequences/widgets/searchable_select_multiple.html"

    def __init__(self, attrs: dict[str, str] | None = None) -> None:
        """Initialize searchable multi-select widget with default class."""
        default_attrs = {"class": "searchable-select-multiple"}
        if attrs:
            default_attrs.update(attrs)
        super().__init__(attrs=default_attrs)


class ExtraDataCreateForm(forms.ModelForm):
    """
    Form for creating ExtraData instances.

    This form includes fields for all attributes of the ExtraData model,
    allowing users to input data for each field.
    It uses crispy forms for better layout and styling.
    """

    default_renderer: ClassVar = CustomFormRenderer()

    class Meta:
        """Meta configuration for ExtraDataCreateForm."""

        model = ExtraData
        fields = (
            "data_type",
            "name",
            "name_full",
            "name_display",
            "namespace",
            "version",
            "hash_sha256",
            "data_json",
            "media_type",
            "iri",
            "url",
            "description",
            "image",
            "file",
        )

        text_fields_required: ClassVar = ("name",)

        text_fields: ClassVar = (
            "name_full",
            "name_display",
            "version",
            "hash_sha256",
        )

        textarea_fields: ClassVar = (
            "data_json",
            "description",
        )

        widgets: ClassVar = {
            **{
                field: forms.TextInput(attrs={"class": "input input-sm input-bordered w-full", "required": True})
                for field in text_fields_required
            },
            **{
                field: forms.TextInput(attrs={"class": "input input-sm input-bordered w-full"}) for field in text_fields
            },
            **{
                field: forms.Textarea(attrs={"class": "textarea textarea-sm textarea-bordered w-full", "rows": 3})
                for field in textarea_fields
            },
            "data_type": forms.Select(attrs={"class": "select select-sm select-bordered w-full"}),
            "namespace": forms.Select(attrs={"class": "select select-sm select-bordered w-full"}),
            "media_type": forms.Select(attrs={"class": "select select-sm select-bordered w-full"}),
            "iri": forms.URLInput(attrs={"class": "input input-sm input-bordered w-full"}),
            "url": forms.URLInput(attrs={"class": "input input-sm input-bordered w-full"}),
            "image": forms.ClearableFileInput(attrs={"class": "file-input file-input-sm file-input-bordered w-full"}),
            "file": forms.ClearableFileInput(attrs={"class": "file-input file-input-sm file-input-bordered w-full"}),
        }


class ExtraDataUpdateForm(forms.ModelForm):
    """
    Form for updating ExtraData instances.

    This form includes the same fields as the create form but with
    an update submit button.
    """

    default_renderer: ClassVar = CustomFormRenderer()

    class Meta:
        """Meta configuration for ExtraDataUpdateForm."""

        model = ExtraData
        fields = (
            "data_type",
            "name",
            "name_full",
            "name_display",
            "namespace",
            "version",
            "hash_sha256",
            "data_json",
            "media_type",
            "iri",
            "url",
            "description",
            "image",
            "file",
        )

        text_fields_required: ClassVar = ("name",)

        text_fields: ClassVar = (
            "name_full",
            "name_display",
            "version",
            "hash_sha256",
        )

        textarea_fields: ClassVar = (
            "data_json",
            "description",
        )

        widgets: ClassVar = {
            **{
                field: forms.TextInput(attrs={"class": "input input-sm input-bordered w-full", "required": True})
                for field in text_fields_required
            },
            **{
                field: forms.TextInput(attrs={"class": "input input-sm input-bordered w-full"}) for field in text_fields
            },
            **{
                field: forms.Textarea(attrs={"class": "textarea textarea-sm textarea-bordered w-full", "rows": 3})
                for field in textarea_fields
            },
            "data_type": forms.Select(attrs={"class": "select select-sm select-bordered w-full"}),
            "namespace": forms.Select(attrs={"class": "select select-sm select-bordered w-full"}),
            "media_type": forms.Select(attrs={"class": "select select-sm select-bordered w-full"}),
            "iri": forms.URLInput(attrs={"class": "input input-sm input-bordered w-full"}),
            "url": forms.URLInput(attrs={"class": "input input-sm input-bordered w-full"}),
            "image": forms.ClearableFileInput(attrs={"class": "file-input file-input-sm file-input-bordered w-full"}),
            "file": forms.ClearableFileInput(attrs={"class": "file-input file-input-sm file-input-bordered w-full"}),
        }


class SequenceClassCreateForm(forms.ModelForm):
    """Form for creating SequenceClass instances."""

    default_renderer: ClassVar = CustomFormRenderer()

    class Meta:
        """Meta configuration for SequenceClassCreateForm."""

        model = SequenceClass
        fields = ("name", "encoding", "iri", "description")

        text_fields_required: ClassVar = ("name",)

        textarea_fields: ClassVar = (
            "encoding",
            "description",
        )

        widgets: ClassVar = {
            **{
                field: forms.TextInput(attrs={"class": "input input-sm input-bordered w-full", "required": True})
                for field in text_fields_required
            },
            **{
                field: forms.Textarea(attrs={"class": "textarea textarea-sm textarea-bordered w-full", "rows": 3})
                for field in textarea_fields
            },
            "iri": forms.URLInput(attrs={"class": "input input-sm input-bordered w-full"}),
        }


class SequenceClassUpdateForm(forms.ModelForm):
    """Form for updating SequenceClass instances."""

    default_renderer: ClassVar = CustomFormRenderer()

    class Meta:
        """Meta configuration for SequenceClassUpdateForm."""

        model = SequenceClass
        fields = ("name", "encoding", "iri", "description")

        text_fields_required: ClassVar = ("name",)

        textarea_fields: ClassVar = (
            "encoding",
            "description",
        )

        widgets: ClassVar = {
            **{
                field: forms.TextInput(attrs={"class": "input input-sm input-bordered w-full", "required": True})
                for field in text_fields_required
            },
            **{
                field: forms.Textarea(attrs={"class": "textarea textarea-sm textarea-bordered w-full", "rows": 3})
                for field in textarea_fields
            },
            "iri": forms.URLInput(attrs={"class": "input input-sm input-bordered w-full"}),
        }


class SequenceCreateForm(forms.ModelForm):
    """Form for creating Sequence instances."""

    default_renderer: ClassVar = CustomFormRenderer()

    class Meta:
        """Meta configuration for SequenceCreateForm."""

        model = Sequence
        fields = (
            "namespace",
            "name_display",
            "name",
            "hash_sha256",
            "hash_blockchain",
            "sequence_class",
            "sequence",
            "properties",
            "uniprot_accession",
            "ncbi_gi",
            "ncbi_accession_version",
            "source_organism",
            "media_type",
            "sequence_file",
            "status",
            "version",
            "tags",
            "description",
            "iri",
            "datetime_last_modified",
        )

        text_fields_required: ClassVar = ("name",)

        text_fields: ClassVar = (
            "name_display",
            "hash_sha256",
            "hash_blockchain",
            "uniprot_accession",
            "ncbi_gi",
            "ncbi_accession_version",
            "version",
        )

        textarea_fields: ClassVar = (
            "sequence",
            "properties",
            "source_organism",
            "description",
        )

        widgets: ClassVar = {
            **{
                field: forms.TextInput(attrs={"class": "input input-sm input-bordered w-full", "required": True})
                for field in text_fields_required
            },
            **{
                field: forms.TextInput(attrs={"class": "input input-sm input-bordered w-full"}) for field in text_fields
            },
            **{
                field: forms.Textarea(attrs={"class": "textarea textarea-sm textarea-bordered w-full", "rows": 3})
                for field in textarea_fields
            },
            "namespace": forms.Select(attrs={"class": "select select-sm select-bordered w-full"}),
            "sequence_class": forms.Select(attrs={"class": "select select-sm select-bordered w-full"}),
            "media_type": forms.Select(attrs={"class": "select select-sm select-bordered w-full"}),
            "status": forms.Select(attrs={"class": "select select-sm select-bordered w-full"}),
            "tags": SearchableSelectMultiple(),
            "sequence_file": forms.ClearableFileInput(
                attrs={"class": "file-input file-input-sm file-input-bordered w-full"}
            ),
            "iri": forms.URLInput(attrs={"class": "input input-sm input-bordered w-full"}),
            "datetime_last_modified": forms.DateTimeInput(
                attrs={"class": "input input-sm input-bordered w-full", "type": "datetime-local"}
            ),
        }


class SequenceUpdateForm(forms.ModelForm):
    """Form for updating Sequence instances."""

    default_renderer: ClassVar = CustomFormRenderer()

    class Meta:
        """Meta configuration for SequenceUpdateForm."""

        model = Sequence
        fields = (
            "namespace",
            "name_display",
            "name",
            "hash_sha256",
            "hash_blockchain",
            "sequence_class",
            "sequence",
            "properties",
            "uniprot_accession",
            "ncbi_gi",
            "ncbi_accession_version",
            "source_organism",
            "media_type",
            "sequence_file",
            "status",
            "version",
            "tags",
            "description",
            "iri",
            "datetime_last_modified",
        )

        text_fields_required: ClassVar = ("name",)

        text_fields: ClassVar = (
            "name_display",
            "hash_sha256",
            "hash_blockchain",
            "uniprot_accession",
            "ncbi_gi",
            "ncbi_accession_version",
            "version",
        )

        textarea_fields: ClassVar = (
            "sequence",
            "properties",
            "source_organism",
            "description",
        )

        widgets: ClassVar = {
            **{
                field: forms.TextInput(attrs={"class": "input input-sm input-bordered w-full", "required": True})
                for field in text_fields_required
            },
            **{
                field: forms.TextInput(attrs={"class": "input input-sm input-bordered w-full"}) for field in text_fields
            },
            **{
                field: forms.Textarea(attrs={"class": "textarea textarea-sm textarea-bordered w-full", "rows": 3})
                for field in textarea_fields
            },
            "namespace": forms.Select(attrs={"class": "select select-sm select-bordered w-full"}),
            "sequence_class": forms.Select(attrs={"class": "select select-sm select-bordered w-full"}),
            "media_type": forms.Select(attrs={"class": "select select-sm select-bordered w-full"}),
            "status": forms.Select(attrs={"class": "select select-sm select-bordered w-full"}),
            "tags": SearchableSelectMultiple(),
            "sequence_file": forms.ClearableFileInput(
                attrs={"class": "file-input file-input-sm file-input-bordered w-full"}
            ),
            "iri": forms.URLInput(attrs={"class": "input input-sm input-bordered w-full"}),
            "datetime_last_modified": forms.DateTimeInput(
                attrs={"class": "input input-sm input-bordered w-full", "type": "datetime-local"}
            ),
        }



"""
_____________________________________________________________________.

:PROJECT: LARAsuite

*lara_django_sequences views *

:details: lara_django_sequences views module.
         - add app specific urls here

:authors: mark doerr <mark.doerr@uni-greifswald.de>

.. note:: -
.. todo:: -
________________________________________________________________________
"""

from __future__ import annotations

from dataclasses import dataclass, field
from typing import Any

from django.views.generic import CreateView, DeleteView, DetailView, UpdateView
from django_tables2 import SingleTableView

from .forms import SequenceCreateForm, SequenceUpdateForm
from .models import Sequence
from .tables import SequenceTable

# Create your lara_django_sequences views here.


@dataclass
class SequencesMenu:
    """Navigation menu for sequence-related views."""

    menu_items: list[dict] = field(
        default_factory=lambda: [
            {"name": "Sequences", "path": "lara_django_sequences:sequence-list"},
        ]
    )


class SequenceSingleTableView(SingleTableView):
    """View for displaying a list of sequences in a table."""

    model = Sequence
    table_class = SequenceTable

    # fields = ('name', 'name_full', 'URL', 'pid', 'iri', 'manufacturer', 'model_no', 'product_type', 'type_barcode',
    # 'product_no', 'weight', 'specifications', 'spec_json', 'spec_doc', 'brochure', 'quickstart', 'manual',
    # 'service_manual', 'icon', 'image', 'description', 'sequence_id', 'sequence_class', 'shape')

    template_name = "lara_django_sequences/list.html"
    success_url = "/sequences/sequence/list"

    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        """Get context data for the template."""
        context = super().get_context_data(**kwargs)
        context["section_title"] = "Sequence - List"
        context["create_link"] = "lara_django_sequences:sequence-create"
        context["menu_items"] = SequencesMenu().menu_items
        return context


class SequenceDetailView(DetailView):
    """View for displaying detailed information about a sequence."""

    model = Sequence

    template_name = "lara_django_sequences/sequence_detail.html"

    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        """Get context data for the template."""
        context = super().get_context_data(**kwargs)
        context["section_title"] = "Sequence - Details"
        context["update_link"] = "lara_django_sequences:sequence-update"
        context["menu_items"] = SequencesMenu().menu_items
        return context


class SequenceCreateView(CreateView):
    """View for creating a new sequence."""

    model = Sequence

    template_name = "lara_django_sequences/create_form.html"
    form_class = SequenceCreateForm
    success_url = "/sequences/sequence/list"

    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        """Get context data for the template."""
        context = super().get_context_data(**kwargs)
        context["section_title"] = "Sequence - Create"
        return context


class SequenceUpdateView(UpdateView):
    """View for updating an existing sequence."""

    model = Sequence

    template_name = "lara_django_sequences/update_form.html"
    form_class = SequenceUpdateForm
    success_url = "/sequences/sequence/list"

    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        """Get context data for the template."""
        context = super().get_context_data(**kwargs)
        context["section_title"] = "Sequence - Update"
        context["delete_link"] = "lara_django_sequences:sequence-delete"
        return context


class SequenceDeleteView(DeleteView):
    """View for deleting a sequence."""

    model = Sequence

    template_name = "lara_django_sequences/delete_form.html"
    success_url = "/sequences/sequence/list"

    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        """Get context data for the template."""
        context = super().get_context_data(**kwargs)
        context["section_title"] = "Sequence - Delete"
        context["delete_link"] = "lara_django_sequences:sequence-delete"
        return context



"""
_____________________________________________________________________.

:PROJECT: LARAsuite

*lara_django_data filter app *

:details: lara_django_data filter app.
         -
:authors: mark doerr <mark.doerr@uni-greifswald.de>

.. note:: -
.. todo:: -
________________________________________________________________________
"""

from django_filters.rest_framework import FilterSet

from .models import ExtraData, Sequence, SequenceClass, SequencesSubset


class ExtraDataFilterSet(FilterSet):
    """FilterSet for ExtraData model."""

    class Meta:  # noqa: D106
        model = ExtraData
        fields = {  # noqa: RUF012
            "name_display": ["exact", "contains"],
            "name": ["exact", "contains"],
            "iri": ["exact", "contains"],
            "url": ["exact", "contains"],
            "datetime_created": ["lt", "gt", "exact", "range"],
            "datetime_last_modified": ["lt", "gt", "exact", "range"],
            "description": ["exact", "contains"],
        }


class SequenceClassFilterSet(FilterSet):
    """FilterSet for SequenceClass model."""

    class Meta:  # noqa: D106
        model = SequenceClass
        fields = {  # noqa: RUF012
            "name": ["exact", "contains"],
            "iri": ["exact", "contains"],
            "description": ["exact", "contains"],
        }


class SequenceFilterSet(FilterSet):
    """FilterSet for Sequence model."""

    class Meta:  # noqa: D106
        model = Sequence
        fields = {  # noqa: RUF012
            "name": ["exact", "contains"],
            "name_display": ["exact", "contains"],
            "iri": ["exact", "contains"],
            "version": ["exact", "contains"],
            # "sequence": ["exact", "contains"],
            "datetime_last_modified": ["lt", "gt", "exact", "range"],
            "description": ["exact", "contains"],
        }


class SequencesSubsetFilterSet(FilterSet):
    """FilterSet for SequencesSubset model."""

    class Meta:  # noqa: D106
        model = SequencesSubset
        fields = {  # noqa: RUF012
            "name": ["exact", "contains"],
            "datetime_last_modified": ["lt", "gt", "exact", "range"],
            "description": ["exact", "contains"],
        }



"""
_____________________________________________________________________.

:PROJECT: LARAsuite

*lara_django_sequences app *

:details: lara_django_sequences app configuration.
         This provides a generic django app configuration mechanism.
         For more details see:
         https://docs.djangoproject.com/en/4.0/ref/applications/
         -
:authors: mark doerr <mark.doerr@uni-greifswald.de>

.. note:: -
.. todo:: -
________________________________________________________________________
"""

from django.apps import AppConfig


class LaraDjangoSequencesConfig(AppConfig):
    """
    Configuration class for the LARA-django Sequences app.

    This class defines the app's name, URL path, verbose name,
    icon, color, and description.
    It also specifies the apps that will be included in the LARA instance.
    """

    name = "lara_django_sequences"
    url_path = "sequences/"
    # enter a verbose name for your app: lara_django_sequences here - this will be used in the admin interface
    verbose_name = "LARA-django Sequences"
    lara_app_icon = (
        "lara_django_sequences_icon.svg"  # this will be used to display an icon, e.g. in the main LARA menu.
    )
    lara_app_color = "indigo"  # this will be used to highlight the app in the main LARA sidebar and app page
    verbose_name_short = "Sequences"
    lara_app_description = "Sequences and Sequence Sets"
    lara_class_apps = (
        {
            "name": "Sequences",
            "path": "lara_django_people:entity-list",  # "/projects/project/list",
            "icon": "lara_projects_icon.svg",
            "color": lara_app_color,
        },
        {
            "name": "Sequence Sets",
            "path": "lara_django_people:entity-list",  # "/projects/project/list",
            "icon": "lara_projects_icon.svg",
            "color": lara_app_color,
        },
    )
    lara_instance_apps = ()

    def ready(self):
        """
        Ready method for the app configuration.

        This method is called when the app is ready to be used.

        :TODO: add urlpatterns
        It includes the app's URLs in the main LARA URL configuration.
        """
        from django.urls import include, path
        from lara_django.urls import urlpatterns

        urlpatterns += [
            path(self.url_path, include("lara_django_sequences.urls", namespace="lara_django_sequences")),
        ]
